@@ -5,6 +5,7 @@
  * License or the Perl Artistic License.
  */
 #include <config.h>
+#include <errno.h>
 #include <stdio.h>
 #include <string.h>
 #include <time.h>
@@ -1259,6 +1260,7 @@ void SLtt_get_terminfo (void)
    static int been_here;
 
 #ifdef TIOCGWINSZ
+#ifndef SCO_FLAVOR
    struct winsize size;
 
    if (ioctl (0, TIOCGWINSZ, &size) == 0){
@@ -1267,6 +1269,7 @@ void SLtt_get_terminfo (void)
 	   SLtt_Screen_Rows = size.ws_row;
        }
    }
+#endif
 #endif
    
    if (been_here)
