@@ -96,13 +96,21 @@ WPanel *get_a_panel (WPanel *panel)
 	return (WPanel *) get_panel_widget (get_other_index ());
 }
 
-int view_file (char *filename, int normal, int internal)
+/* view_file (filename, normal, internal)
+ *
+ * Inputs:
+ *   filename:   The file name to view
+ *   plain_view: If set does not do any fancy pre-processing (no filtering) and
+ *               always invokes the internal viewer.
+ *   internal:   If set uses the internal viewer, otherwise an external viewer.
+ */
+int view_file (char *filename, int plain_view, int internal)
 {
     static char *viewer = 0;
     int move_dir = 0;
 
 
-    if (normal) {
+    if (plain_view) {
         int changed_hex_mode = 0;
         int changed_nroff_flag = 0;
     
@@ -128,16 +136,35 @@ int view_file (char *filename, int normal, int internal)
 	    repaint_screen ();
 	}
     } else {
+	char *localcopy;
+	
 	if (!viewer){
 	    viewer = getenv ("PAGER");
 	    if (!viewer)
 		viewer = "view";
 	}
-	execute_internal (viewer, filename);
+	/* The file may be a non local file, get a copy */
+	if (!vfs_file_is_local (filename)){
+	    localcopy = mc_getlocalcopy (filename);
+	    if (localcopy == NULL){
+		message (1, " Error ", " Can not fetch a local copy of %s ", filename);
+		return 0;
+	    }
+	    execute_internal (viewer, localcopy);
+	    mc_ungetlocalcopy (filename, localcopy, 0);
+	} else 
+	    execute_internal (viewer, filename);
     }
     return move_dir;
 }
 
+/* scan_for_file (panel, idx, direction)
+ *
+ * Inputs:
+ *   panel:     pointer to the panel on which we operate
+ *   idx:       starting file.
+ *   direction: 1, or -1
+ */
 static int scan_for_file (WPanel *panel, int idx, int direction)
 {
     int i = idx + direction;
@@ -154,7 +181,8 @@ static int scan_for_file (WPanel *panel, int idx, int direction)
     return i;
 }
 
-static void view_cmd_internal (WPanel *panel, int normal)
+/* do_view: Invoked as the F3/F13 key. */
+static void do_view_cmd (WPanel *panel, int normal)
 {
     int dir, file_idx;
     panel = get_a_panel (panel);
@@ -168,7 +196,7 @@ static void view_cmd_internal (WPanel *panel, int normal)
 		return;
 	    }
 	}
-	do_cd (selection (panel)->fname);
+	do_cd (selection (panel)->fname, cd_exact);
 	return;
 	
     }
@@ -188,12 +216,12 @@ static void view_cmd_internal (WPanel *panel, int normal)
 
 void view_cmd (WPanel *panel)
 {
-    view_cmd_internal (panel, 0);
+    do_view_cmd (panel, 0);
 }
 
 void view_simple_cmd (void)
 {
-    view_cmd_internal (cpanel, 1);
+    do_view_cmd (cpanel, 1);
 }
 
 void filtered_view_cmd (WPanel *panel)
@@ -537,6 +565,7 @@ void ext_cmd (void)
 	free (buffer);
     } else if (dir == 1)
 	do_edit (LIBDIR "mc.ext");
+   flush_extension_file ();
 }
 
 void menu_edit_cmd (void)
@@ -583,7 +612,7 @@ void quick_chdir_cmd (void)
     if (get_current_type () == view_tree)
 	tree_chdir (the_tree, target);
     else
-	do_cd (target);
+	do_cd (target, cd_exact);
 }
 
 #ifdef USE_VFS
@@ -595,7 +624,7 @@ void reselect_vfs (void)
     if (!target)
 	return;
 
-    do_cd (target);
+    do_cd (target, cd_exact);
     free (target);
 }
 #endif
@@ -897,7 +926,11 @@ static void do_link (int symbolic_link)
         if ( ! ((s[0] == '/') && (s[1] == 0)))
             strcat(s, "/");
         strcat(s, selection (cpanel)->fname);
-        strcpy(d, opanel->cwd);
+	if (get_other_type () == view_listing)
+	    strcpy(d, opanel->cwd);
+	else
+	    strcpy (d,"");
+	
         if ( ! ((d[0] == '/') && (d[1] == 0)))
             strcat(d, "/");
         symlink_dialog (s, d, &dest, &src);
@@ -975,10 +1008,12 @@ void other_symlink_cmd (void)
         p = copy_strings (cpanel->cwd, PATH_SEP_STR, selection (cpanel)->fname, NULL);
     else
         p = copy_strings (cpanel->cwd, selection (cpanel)->fname, NULL);
+    
     if (opanel->cwd [strlen (opanel->cwd) - 1] != PATH_SEP)
-        r = copy_strings (opanel->cwd, PATH_SEP_STR, selection (cpanel)->fname, NULL);
+	r = copy_strings (opanel->cwd, PATH_SEP_STR, selection (cpanel)->fname, NULL);
     else
-        r = copy_strings (opanel->cwd, selection (cpanel)->fname, NULL);
+	r = copy_strings (opanel->cwd, selection (cpanel)->fname, NULL);
+    
     q = copy_strings (" Link symbolically ", name_trunc (p, 32), " to:", NULL);
     dest = input_expand_dialog (" Relative symlink ", q, r);
     if (dest) {
@@ -1106,7 +1141,7 @@ static void nice_cd (char *text, char *xtext, char *help, char *prefix, int to_h
 	return;
 
     cd_path = copy_strings (prefix, machine, to_home ? "/~/" : NULL, NULL);
-    do_panel_cd (MENU_PANEL, cd_path);
+    do_panel_cd (MENU_PANEL, cd_path, 0);
     free (cd_path);
     free (machine);
 }
