@@ -41,8 +41,25 @@
 #include "../vfs/vfs.h"
 
 /* "$Id: ext.c,v 1.10 1995/02/21 19:05:45 miguel Exp $" */
+
+/* If set, we execute the file command to check the file type */
 int use_file_to_check_type = 1;
 
+/* This variable points to a copy of the mc.ext file in memory
+ * With this we avoid loading/parsing the file each time we 
+ * need it 
+ */
+static char *data = NULL;
+
+void flush_extension_file (void)
+{
+    if (data){
+        free (data);
+        data = NULL;
+    }
+   
+}
+
 void exec_extension (char *filename, char *data, char *drops, int *move_dir)
 {
     char *file_name;
@@ -200,12 +217,15 @@ void exec_extension (char *filename, char *data, char *drops, int *move_dir)
     	while (*q && *q != ' ' && *q != '\t')
     	    q++;
     	*q = 0;
-    	do_cd (p);
+    	do_cd (p, cd_parse_command);
     } else {
+	shell_execute (file_name, 1);
+#ifdef OLD_CODE
 	if (vfs_current_is_local ())
 	    shell_execute (file_name, 1);
 	else
 	    message (1, " Warning ", " Can't execute commands on a Virtual File System directory ");
+#endif
     }
     unlink (file_name);
     if (localcopy) {
@@ -238,7 +258,6 @@ void exec_extension (char *filename, char *data, char *drops, int *move_dir)
  * icon name and we set the variable regex_command_title to Title for
  * that icon.
  */
-static char *data = NULL;
 char *regex_command_title = NULL;
 char *regex_command (char *filename, char *action, char *drops, int *move_dir)
 {
@@ -489,11 +508,23 @@ match_file_output:
     	                    break;
     	            } else if (!strcmp (action, p)) {
     	                *r = c;
-    	                for (p = r + 1; *p == ' ' || *p == '\t'; p++);
-    	                if (p < q) { /* Empty commands just stop searching
-    	                              * through, they don't do anything
-    	                              */
-    	                    exec_extension (filename, r + 1, drops, move_dir);
+    	                for (p = r + 1; *p == ' ' || *p == '\t'; p++)
+			    ;
+
+			/* Empty commands just stop searching
+			 * through, they don't do anything
+			 *
+			 * We need to copy the filename because exec_extension
+			 * may end up invoking update_panels thus making the
+			 * filename parameter invalid (ie, most of the time,
+			 * we get filename as a pointer from cpanel->dir).
+			 */
+    	                if (p < q) { 
+			    char *filename_copy = strdup (filename);
+			    
+    	                    exec_extension (filename_copy, r + 1, drops, move_dir);
+			    free (filename_copy);
+			    
     	                    to_return = "Success";
     	                }
     	                break;
