@@ -112,14 +112,22 @@ struct re_pattern_buffer rx;
 struct re_registers regs;
 static char *dest_mask = NULL;
 static int op_follow_symlinks = 0;
+
+/* Recursive operation on subdirectories */
 int dive_into_subdirs = 1;
-int stable_symlinks = 0;
+
+/* If running as root, preserve the original uid/gid on file copy */
 int preserve_uidgid = 1;
 
+int stable_symlinks = 0;
+
 #define WX 60
 #define WY 10
 #define BY 10
 
+#define DIF_TIME(t1,t2) ((t2.tv_sec -t1.tv_sec) *1000+ \
+			 (t2.tv_usec-t1.tv_usec)/1000)
+
 static Dlg_head *op_dlg;
 static char *op_names [] = { " Copy ", " Move ", " Delete " };
 static int selected_button;
@@ -142,6 +150,7 @@ enum { RECURSIVE_YES, RECURSIVE_NO, RECURSIVE_ALWAYS,
 static int recursive_result;
 static int recursive_erase (char *s);
 
+/* Describe the components in the panel operations window */
 static WLabel *FileLabel [2];
 static WLabel *FileString [2];
 static WLabel *ProgressLabel [3];
@@ -661,7 +670,7 @@ int convert_case (int c, enum CaseConvs *conversion)
 }
 
 static int transform_error = 0;
-static char *transform_source (char *source)
+static char *do_transform_source (char *source)
 {
     int j, k, l, len;
     char *fnsource = x_basename (source);
@@ -731,6 +740,22 @@ static char *transform_source (char *source)
     return fntarget;
 }
 
+static char *transform_source (char *source)
+{
+    char *s = strdup (source);
+    char *q;
+
+    /* We remove \n from the filename since regex routines would use \n as an anchor */
+    /* this is just to be allowed to maniupulate file names with \n on it */
+    for (q = s; *q; q++){
+	if (*q == '\n')
+	    *q = ' ';
+    }
+    q = do_transform_source (s);
+    free (s);
+    return q;
+}
+
 void free_linklist (void)
 {
     struct link *lp, *lp2;
@@ -960,7 +985,11 @@ int copy_file_file (char *src_path, char *dst_path, int toplevel,
 	do_append = 0;
 	return return_status;
     }
+
+#ifndef _OS_NT
+    /* Only unix uses the vfs ctls, vfs does not work on NT yet */
 #define USE_VFS_CTLS 1
+#endif
 #ifdef USE_VFS_CTLS
     /* Before we can use the mc_ctl routine, the ftpfs code should be
      * changed so that it aborts connections as it should
@@ -970,6 +999,7 @@ int copy_file_file (char *src_path, char *dst_path, int toplevel,
     do_remote_copy = mc_ctl (source_desc, MCCTL_ISREMOTECOPY, 0);
 #else
 #define do_remote_copy 0
+#define mc_ctl(a,b,c) 0
 #endif
     
     if (!do_remote_copy) {
@@ -1080,7 +1110,10 @@ int copy_file_file (char *src_path, char *dst_path, int toplevel,
         }
     else {
         int i, size;
+	struct timeval tv_last_update;
+	struct timeval tv_current;
 	
+	gettimeofday (&tv_last_update, (struct timezone *) NULL);
         for (i = 1; i;) {
             switch (size = mc_ctl (source_desc, MCCTL_REMOTECOPYCHUNK, 8192)) {
                 case MCERR_TARGETOPEN:
@@ -1115,6 +1148,14 @@ int copy_file_file (char *src_path, char *dst_path, int toplevel,
 		       |(S_IXGRP|S_IWGRP|S_IRGRP)))) /* group */
 		    src_mode = S_IRUSR|S_IWUSR|S_IROTH|S_IRGRP;
 	    }
+
+	    /* Update rotating dash in case it's been a while since we updated it */
+	    /* Hardcoded to 2000 miliseconds */
+	    gettimeofday (&tv_current, NULL);
+	    if (DIF_TIME (tv_last_update, tv_current) > 2000){
+		rotate_dash ();
+		tv_last_update = tv_current;
+	    }
 	    return_status = show_file_progress (n_read_total, file_size);
 	    refresh ();
 	    if (return_status != FILE_CONT)
@@ -1630,9 +1671,8 @@ static int is_wildcarded (char *p)
     return 0;
 }
 
-#if BACKGROUND
+#ifdef BACKGROUND
 
-int attention = 0;
 int we_are_background = 0;
 
 int
@@ -1641,7 +1681,9 @@ background_attention (int fd)
     char c;
     
     read (fd, &c, 1);
-    attention = 1;
+    if (c == ATTENTION){
+	
+    }
 }
 
 int background_processor (int fd)
@@ -1671,7 +1713,7 @@ static int
 do_background ()
 {
     int fds [2];		/* stdin/stdout streams */
-    int comm [2];		/* data connection stream */
+    int comm [2];		/* control connection stream */
     int pid;
     
     if (socketpair (AF_UNIX, SOCK_STREAM, 0, fd) == -1)
@@ -1747,7 +1789,7 @@ char *file_mask_dialog (char *header, char *text, char *def_text, int only_one)
     { quick_button, 34, 64, 9, FMDY, "[ Cancel ]", 'C', 2, B_CANCEL, 0, 0, 0,
 	  "b." },
     { quick_button, 20, 64, 9, FMDY, "[ Ok ]", 'O', 2, B_ENTER, 0, 0, 0, "" },
-#if 0
+#ifdef BACKGROUND
     { quick_button, 48, 64, 9, FMDY, "[ & ]", '&', 2, B_USER, 0, 0, 0, 0, "" },
 #endif
     { 0 } };
@@ -1779,7 +1821,7 @@ ask_file_mask:
     if ((val = quick_dialog_skip (&Quick_input, SKIP)) == B_CANCEL)
 	return 0;
 
-#if BACKGROUND
+#ifdef BACKGROUND
     /* If it is a background operation */
     if (val == B_USER){
 	if (do_background () != -1)
@@ -1800,6 +1842,7 @@ ask_file_mask:
 	source_mask = convert_pattern (source_mask, match_file, 1);
 	source_easy_patterns = easy_patterns;
     }
+
     error = re_compile_pattern (source_mask, strlen (source_mask), &rx);
     if (error) {
 	message (1, " Error ", "Invalid source pattern \"%s\" \n %s ",
