@@ -632,11 +632,11 @@ void do_find (void)
 	if (v == B_ENTER){
 	    if (dirname || filename){
 		if (dirname){
-		    do_cd (dirname);
+		    do_cd (dirname, cd_exact);
 		    if (filename)
 			try_to_select (current_panel, filename + 4);
 		} else if (filename)
-		    do_cd (filename);
+		    do_cd (filename, cd_exact);
 		paint_panel (current_panel);
 		select_item (cpanel);
 	    }
