@@ -910,7 +910,7 @@ int is_idle (void)
     /* Check for incoming key presses     *
      * If there are any we say we're busy */
 
-    struct fd_set select_set;
+    fd_set select_set;
     struct timeval timeout;
     FD_ZERO (&select_set);
     FD_SET (0, &select_set);
@@ -923,7 +923,7 @@ int is_idle (void)
 int ctrl_pressed ()
 {
 #ifdef __linux__
-    int modifiers;
+    unsigned char modifiers;
 
     modifiers = 6;
 
