@@ -680,6 +680,26 @@ void print_vfs_message(char *msg, ...)
     }
 }
 
+#ifdef BACKGROUND
+void set_proc_stat_msg (char *msg)
+{
+    int l = strlen (msg);
+
+    process_status->widget.cols = l;
+    process_status->x = COLS - l;
+    label_set_text (process_status, msg);
+
+    /* since the message has changed, and we are called from one of the 
+     * get_event channels, the message updating does not take place
+     * automatically: force a cursor update and a screen refresh
+     */
+    if (current_dlg == midnight_dlg){
+	update_cursor (midnight_dlg);
+	refresh ();
+    }
+}
+#endif
+
 void rotate_dash (void)
 {
 #ifndef HAVE_X
