@@ -50,7 +50,7 @@ typedef struct {
 } mad_mem_area;
 
 static mad_mem_area mem_areas [MAD_MAX_AREAS];
-static void *watch_free_pointer = 0;
+void *watch_free_pointer = 0;
 
 /* This function is only called by the mad_check function */
 static void mad_abort (char *message, int area, char *file, int line)
