@@ -21,7 +21,6 @@
 #include <config.h>
 #ifdef _OS_NT
 #    include <windows.h>
-#    define baudrate 19200
 #endif
 
 #include "tty.h"
@@ -215,6 +214,11 @@ WLabel *the_prompt;
 /* The hint bar */
 WLabel *the_hint;
 
+#ifdef BACKGROUND
+/* The foreground/background message */
+WLabel *process_status;
+#endif
+
 /* The button bar */
 WButtonBar *the_bar;
 #ifdef HAVE_X
@@ -374,22 +378,21 @@ void reload_panelized (WPanel *panel)
     
     if (panel != current_panel)
 	mc_chdir (panel->cwd);
-    
+
     for (i = 0, j = 0; i < panel->count; i++){
-	if (mc_lstat (list->list [i].fname, &list->list [i].buf))
+	if (mc_lstat (list->list [i].fname, &list->list [i].buf)){
+	    free (list->list [i].fname);
 	    continue;
+	}
 	if (j != i)
 	    list->list [j] = list->list [i];
 	j++;
     }
     if (j == 0)
 	panel->count = set_zero_dir (list);
-    else {
-	for (i = j; i < panel->count; i++){
-	    free (list->list [i].fname);
-	}
+    else
 	panel->count = j;
-    }
+
     if (panel != current_panel)
 	mc_chdir (current_panel->cwd);
 }
@@ -593,7 +596,7 @@ static void parse_control_file (void)
 		change_panel ();
 		change = 1;
 	    }
-	    do_cd (current);
+	    do_cd (current, cd_parse_command);
 	    if (change)
 		change_panel ();
 	} else {
@@ -607,7 +610,9 @@ static void parse_control_file (void)
     paint_panel (cpanel);
     paint_panel (opanel);
 }
-#endif
+#else
+#define parse_control_file()
+#endif /* WANT_PARSE */
 
 void clr_scr (void)
 {
@@ -697,7 +702,7 @@ void save_cwds_stat (void)
 #ifdef HAVE_SUBSHELL_SUPPORT
 void do_possible_cd (char *new_dir)
 {
-    if (!do_cd (new_dir))
+    if (!do_cd (new_dir, cd_exact))
 	message (1, " Warning ",
 		 " The Commander can't change to the directory that \n"
 		 " the subshell claims you are in.  Perhaps you have \n"
@@ -732,7 +737,6 @@ void do_execute (const char *shell, const char *command, int internal_command)
 {
 #ifdef HAVE_SUBSHELL_SUPPORT
     char *new_dir = NULL;
-    char **new_dir_p;
 #ifdef USE_VFS
     char *old_vfs_dir = 0;
 
@@ -757,8 +761,7 @@ void do_execute (const char *shell, const char *command, int internal_command)
 	do_update_prompt ();
 
 	/* We don't care if it died, higher level takes care of this */
-	new_dir_p = vfs_current_is_local () ? &new_dir : NULL;
-	invoke_subshell (command, VISIBLY, new_dir_p);
+	invoke_subshell (command, VISIBLY, &new_dir);
     } else
 #endif
 	my_system (!internal_command, shell, command);
@@ -799,9 +802,7 @@ void do_execute (const char *shell, const char *command, int internal_command)
 
     update_panels (UP_OPTIMIZE, UP_KEEPSEL, UP_KEEPSEL);
     
-#ifdef WANT_PARSE
     parse_control_file ();
-#endif
     unlink (control_file);
     do_refresh ();
     use_dash (TRUE);
@@ -920,7 +921,7 @@ void subshell_chdir (char *directory)
 }
 
 /* Changes the current panel directory */
-int do_panel_cd (WPanel *panel, char *new_dir)
+int do_panel_cd (WPanel *panel, char *new_dir, enum cd_enum cd_type)
 {
     char *directory, *olddir;
     char temp [MC_MAXPATHLEN];
@@ -929,12 +930,14 @@ int do_panel_cd (WPanel *panel, char *new_dir)
 
     /* Convert *new_path to a suitable pathname, handle ~user */
     
-    while (*new_dir == ' ')
-	new_dir++;
-
-    if (!strcmp (new_dir, "-")){
-	strcpy (temp, panel->lwd);
-	new_dir = temp;
+    if (cd_type == cd_parse_command){
+	while (*new_dir == ' ')
+	    new_dir++;
+    
+	if (!strcmp (new_dir, "-")){
+	    strcpy (temp, panel->lwd);
+	    new_dir = temp;
+	}
     }
     directory = *new_dir ? new_dir : home_dir;
 
@@ -968,9 +971,9 @@ int do_panel_cd (WPanel *panel, char *new_dir)
     return 1;
 }
 
-int do_cd (char *new_dir)
+int do_cd (char *new_dir, enum cd_enum exact)
 {
-    return do_panel_cd (cpanel, new_dir);
+    return do_panel_cd (cpanel, new_dir, exact);
 }
 
 #ifdef HAVE_SUBSHELL_SUPPORT
@@ -1022,12 +1025,12 @@ int maybe_cd (int char_code, int move_up_dir)
     if (navigate_with_arrows){
 	if (!input_w (cmdline)->buffer [0]){
 	    if (!move_up_dir){
-		do_cd ("..");
+		do_cd ("..", cd_exact);
 		return 1;
 	    }
 	    if (S_ISDIR (selection (cpanel)->buf.st_mode)
 		|| link_isdir (selection (cpanel))){
-		do_cd (selection (cpanel)->fname);
+		do_cd (selection (cpanel)->fname, cd_exact);
 		return 1;
 	    }
 	}
@@ -1046,7 +1049,7 @@ static void tree_leave (WPanel *p)
 	change_panel ();
 	change = 1;
     }
-    if (!do_cd (dir))
+    if (!do_cd (dir, cd_exact))
 	message (1, " Error ", " Can't chdir to \"%s\" \n %s ",
 		 dir, unix_error_string (errno));
     if (change)
@@ -1064,6 +1067,12 @@ static void quick_cmd_no_menu (void)
     set_display_type (cpanel == left_panel ? 1 : 0, view_quick);
 }
 
+static void switch_to_listing (int panel_index)
+{
+    if (get_display_type (panel_index) != view_listing)
+	set_display_type (panel_index, view_listing);
+}
+
 static void listing_cmd (void)
 {
     int   view_type, use_msformat;
@@ -1082,8 +1091,7 @@ static void listing_cmd (void)
     if (view_type == -1)
 	return;
 
-    if (get_display_type (MENU_PANEL_IDX) != view_listing)
-	set_display_type (MENU_PANEL_IDX, view_listing);
+    switch_to_listing (MENU_PANEL_IDX);
 
     p = MENU_PANEL_IDX == 0 ? left_panel : right_panel;    
     free (p->user_format);
@@ -1094,8 +1102,11 @@ static void listing_cmd (void)
     p->user_mini_status = use_msformat; 
     p->list_type = view_type;
     
-    if (set_panel_format (p, panel_format (p)))
+    if (set_panel_format (p, panel_format (p))){
+	free (p->user_format);
+	p->user_format = strdup (DEFAULT_USER_FORMAT);
 	set_panel_format (p, DEFAULT_USER_FORMAT);
+    }
     do_refresh ();
 }
 
@@ -1114,6 +1125,42 @@ void quick_view_cmd (void)
     set_display_type (MENU_PANEL_IDX, view_quick);
 }
 
+/* Handle the tree internal listing modes switching */
+int set_basic_panel_listing_to (int panel_index, int listing_mode)
+{
+    WPanel *p = (WPanel *) get_panel_widget (panel_index);
+    
+    switch_to_listing (panel_index);
+    p->list_type = listing_mode;
+    if (set_panel_format (p, panel_format (p)))
+	return 0;
+    do_refresh ();
+    return 1;
+}
+
+void toggle_listing_cmd ()
+{
+    int current = get_current_index ();
+    WPanel *p = (WPanel *) get_panel_widget (current);
+    int list_mode = p->list_type;
+    int m;
+    
+    switch (list_mode){
+    case list_full:
+    case list_brief:
+	m = list_long;
+	break;
+    case list_long:
+	m = list_user;
+	break;
+    default:
+	m = list_full;
+    }
+    if (set_basic_panel_listing_to (current, m))
+	return;
+    set_basic_panel_listing_to (current, list_full);
+}
+
 static void sort_cmd (void)
 {
     WPanel  *p;
@@ -1163,41 +1210,6 @@ static void listmode_cmd (void)
 }
 #endif
 
-#ifdef _OS_NT
-/* Hacked from the Antonio Palama's port of MC 1.1 to DOS  */
-void drive_cmd()
-{
-    int d, new_d;
-    char *drive;
-    char buf[5];
-    
-    d = _getdrive();
-    drive = input_dialog(" Enter new drive ", "", "");
-    if (strlen(drive) > 0) {
-	*drive = toupper(*drive);
-	new_d = *drive - 'A' + 1;
-	if (new_d != d)  {
-	    sprintf (buf, "%c:", *drive);
-	    SetCurrentDirectory(buf);           
-            getcwd (cpanel->cwd, sizeof (cpanel->cwd)-2);
-            if (*drive == toupper(*(cpanel->cwd)))  {
-		clean_dir (&cpanel->dir, cpanel->count);
-		cpanel->count = do_load_dir (&cpanel->dir, cpanel->sort_type,
-					     cpanel->reverse, cpanel->filter);
-		cpanel->top_file = 0;
-		cpanel->selected = 0;
-		cpanel->marked = 0;
-		cpanel->total = 0;
-		show_dir(cpanel);
-		reread_cmd();
-            } else
-		message (1, " Error ", " Can't access drive %c: \n", *drive);
-	}
-    }
-   free(drive);
-}
-#endif
-
 static menu_entry PanelMenu [] = {
     { ' ', "Listing mode...",	'L',        listing_cmd },
     { ' ', "Quick view",        'Q',        quick_view_cmd }, 
@@ -1214,7 +1226,7 @@ static menu_entry PanelMenu [] = {
 #endif
     { ' ', "", ' ', 0 },
 #ifdef _OS_NT
-    { ' ', "Drive...  M-d",         ALT('d'),drive_cmd },
+    { ' ', "Drive...  M-d",         ALT('d'),drive_cmd_a },
 #endif
     { ' ', "Rescan    C-r",         XCTRL('R'), reread_cmd }
 };
@@ -1235,7 +1247,7 @@ static menu_entry RightMenu [] = {
 #endif
     { ' ', "", ' ', 0 },
 #ifdef _OS_NT
-    { ' ', "Drive...  M-d",         ALT('d'),drive_cmd },
+    { ' ', "Drive...  M-d",         ALT('d'),drive_cmd_b },
 #endif
     { ' ', "Rescan    C-r",         XCTRL('R'), reread_cmd }
 };
@@ -1247,11 +1259,11 @@ static menu_entry FileMenu [] = {
     { ' ', "Filtered view     M-!",     ALT('!'), filtered_view_cmd },
     { ' ', "Edit               F4",     KEY_F(4), edit_cmd },
     { ' ', "Copy               F5",     KEY_F(5), copy_cmd },
+    { ' ', "Chmod           C-x c",  'C',      chmod_cmd },
+#ifndef _OS_NT
     { ' ', "Link            C-x l",  'L',      link_cmd },
     { ' ', "SymLink         C-x s",  'S',      symlink_cmd },
     { ' ', "edit sYmlink  C-x C-s",  'Y',      edit_symlink_cmd },
-    { ' ', "Chmod           C-x c",  'C',      chmod_cmd },
-#ifndef _OS_NT
     { ' ', "chOwn           C-x o",  'O',      chown_cmd },
     { ' ', "Advanced chown       ",  'A',      chown_advanced_cmd },
 #endif
@@ -1498,7 +1510,14 @@ static void create_panels (void)
     the_hint->transparent = 1;
     the_hint->auto_adjust_cols = 0;
     the_hint->widget.cols = COLS;
-			     
+
+#ifdef BACKGROUND
+    process_status = label_new (0, COLS-12, 0);
+    process_status->transparent = 0;
+    process_status->auto_adjust_cols = 0;
+    process_status->widget.cols = 0;
+#endif
+    
 #ifndef HAVE_XVIEW
     the_menubar = menubar_new (0, 0, COLS, MenuBar, 5);
 #else
@@ -1670,13 +1689,13 @@ static key_map ctl_x_map [] = {
     { 'c',          chmod_cmd },
 #ifndef _OS_NT
     { 'o',          chown_cmd },
-#endif
     { 'l',          link_cmd },
     { XCTRL('l'),   other_symlink_cmd },
     { 's',          symlink_cmd },
     { XCTRL('s'),   edit_symlink_cmd },
     { 'r',          copy_current_readlink },
     { XCTRL('r'),   copy_other_readlink },
+#endif
     { 'i',          info_cmd_no_menu },
     { 'q',          quick_cmd_no_menu },
     { 'h',          add2hotlist_cmd },
@@ -1738,6 +1757,8 @@ static key_map default_map [] = {
     /* Panel refresh */
     { XCTRL('r'), reread_cmd },
 
+    { ALT('t'),   toggle_listing_cmd },
+    
 #ifndef HAVE_XVIEW
     /* Swap panels */
     { XCTRL('u'), swap_cmd },
@@ -1766,8 +1787,9 @@ static void setup_sigwinch ()
 #       ifdef SIGWINCH
             act.sa_handler = flag_winch;
             sigemptyset (&act.sa_mask);
+	    act.sa_flags = 0;
 #           ifdef SA_RESTART
-                act.sa_flags = SA_RESTART;
+                act.sa_flags |= SA_RESTART;
 #           endif
             sigaction (SIGWINCH, &act, &oact);
 #       endif
@@ -2077,7 +2099,10 @@ static void do_nc (void)
 #ifndef HAVE_X
     add_widget (midnight_dlg, the_hint);
     load_hint ();
-#endif
+#ifdef BACKGROUND
+    add_widget (midnight_dlg, process_status);
+#endif /* BACKGROUND */
+#endif /* HAVE_X */
     add_widgetl (midnight_dlg, cmdline, XV_WLAY_RIGHTOF);
     add_widgetl (midnight_dlg, the_prompt, XV_WLAY_DONTCARE);
     tk_end_frame ();
@@ -2318,11 +2343,13 @@ int SLtt_try_termcap;
 	case 't':
 	    SLtt_try_termcap = 1;
 	    break;
-	    
+
+#ifndef _OS_NT
 	case 'N':
 	    SLtt_want_unicode ();
 	    break;
-#endif
+#endif /* _OS_NT */
+#endif /* HAVE_SLANG */
 	    
 	case 'u':
 #ifdef HAVE_SUBSHELL_SUPPORT
@@ -2380,6 +2407,7 @@ int SLtt_try_termcap;
     }
 #ifdef _OS_NT
     SetConsoleTitle ("GNU Midnight Commander");
+    shell = getenv ("COMSPEC");
     if (!shell || !*shell)
 	shell = "cmd.exe";
      /* Default opening mode for files is binary, not text (CR/LF translation) */
@@ -2420,9 +2448,18 @@ int SLtt_try_termcap;
     }
 }
 
+#ifdef _OS_NT
+static void sigchld_handler_no_subshell (int sig)
+{
+}
+
+void init_sigchld (void)
+{
+}
+
+#else
 static void sigchld_handler_no_subshell (int sig)
 {
-#ifndef _OS_NT
     int pid, status;
 
     if (!console_flag)
@@ -2451,7 +2488,6 @@ static void sigchld_handler_no_subshell (int sig)
     }
 
     /* If we get here, some other child exited; ignore it */
-#endif /* OS_NT */
 }
 
 void init_sigchld (void)
@@ -2475,6 +2511,8 @@ void init_sigchld (void)
     sigaction (SIGCHLD, &sigchld_action, NULL);
 }	
 
+#endif /* OS_NT */
+
 int main (int argc, char *argv [])
 {
 #ifndef _OS_NT
@@ -2542,11 +2580,16 @@ int main (int argc, char *argv [])
     
     /* This variable is used by the subshell */
 #ifdef _OS_NT
-    home_dir = copy_strings (getenv ("HOMEDRIVE"), getenv ("HOMEPATH"));
+    home_dir = copy_strings (getenv ("HOMEDRIVE"), getenv ("HOMEPATH"), NULL);
+    /* we need to malloc something (we free(home_dir)) */
+    if (!home_dir) {
+	home_dir = malloc(MAX_PATH);
+	GetWindowsDirectory(home_dir, MAX_PATH);
+    }	
 #else
     home_dir = getenv ("HOME");
-#endif
     home_dir = home_dir ? home_dir : PATH_SEP_STR;
+#endif
 
 #   ifdef HAVE_X
     /* We need this, since ncurses endwin () doesn't restore the signals */
