@@ -122,7 +122,12 @@ void restore_console (void);
    be a repository for all the little auxiliary functions that are used
    elsewhere.  do_panel_cd should be moved to util.c or somewhere. */
 
-int do_cd (char *new_dir);	/* For find.c */
+enum cd_enum {
+    cd_parse_command,
+    cd_exact
+};
+
+int do_cd (char *new_dir, enum cd_enum cd_type);	/* For find.c */
 void change_panel (void);
 void init_sigchld (void);	/* For subshell.c */
 int load_prompt (int fd, void *unused);
@@ -145,6 +150,7 @@ extern WButtonBar *the_bar;
 extern WLabel     *the_prompt;
 extern WLabel     *the_hint;
 extern Dlg_head   *midnight_dlg;
+extern WLabel      *process_status;
 #include "menu.h"
 extern WMenu      *the_menubar;
 #endif
