@@ -229,7 +229,7 @@ static int menubar_handle_key (WMenu *menubar, int key)
     int   i;
 
     /* Lowercase */
-    if (key < 256 && isascii (key)) /* Linux libc.so.5.x.x bug fix */
+    if (key < 256 && ISASCII (key)) /* Linux libc.so.5.x.x bug fix */
 	key = tolower (key);
     
     if (is_abort_char (key)){
@@ -274,7 +274,7 @@ static int menubar_handle_key (WMenu *menubar, int key)
 		continue;
 	    
 	    m = menu->entries [i].hot_key;
-	    if (m < 256 && isascii (m))
+	    if (m < 256 && ISASCII (m))
 	        m = tolower (m);
 	    if (key != m)
 		continue;
