@@ -1416,7 +1416,7 @@ static void prev_page (WPanel *panel)
 static void prev_page_key (WPanel *panel)
 {
     if (console_flag && ctrl_pressed ()){
-	do_cd ("..");
+	do_cd ("..", cd_exact);
     } else
 	prev_page (panel);
 }
@@ -1456,7 +1456,7 @@ static void next_page_key (WPanel *panel)
     if (console_flag&&ctrl_pressed()&&
 	(S_ISDIR(selection (panel)->buf.st_mode) ||
 	 link_isdir (selection (panel))))
-        do_cd (selection (panel)->fname);
+        do_cd (selection (panel)->fname, cd_exact);
     else
 	next_page (panel);
 }
@@ -1651,7 +1651,7 @@ void do_enter (WPanel *panel)
 {
     if (S_ISDIR (selection (panel)->buf.st_mode)
 	|| link_isdir (selection (panel))){
-	do_cd (selection (panel)->fname);
+	do_cd (selection (panel)->fname, cd_exact);
     } else { 
 	if (is_exe (selection (panel)->buf.st_mode) &&
 	    if_link_is_exe (selection (panel))) {
@@ -1693,7 +1693,7 @@ static void chdir_other_panel (WPanel *panel)
 				cpanel->dir.list [cpanel->selected].fname, 0);
 
     change_panel ();
-    do_cd (new_dir);
+    do_cd (new_dir, cd_exact);
     change_panel ();
 
     move_down (panel);
@@ -1739,7 +1739,7 @@ static void chdir_to_readlink (WPanel *panel)
 	}
 
 	change_panel ();
-	do_cd (new_dir);
+	do_cd (new_dir, cd_exact);
 	change_panel ();
 	
 	move_down (panel);
