@@ -59,6 +59,15 @@ static void slang_intr (int signo)
     slinterrupt = 1;
 }
 
+int
+interrupts_enabled (void)
+{
+    struct sigaction current_act;
+
+    sigaction (SIGINT, NULL, &current_act);
+    return current_act.sa_handler == slang_intr;
+}
+
 void enable_interrupt_key(void)
 {
 #ifndef _OS_NT
