@@ -748,6 +748,7 @@ void quote_directory (int fd, char *string)
 	case '\\': q = 1; break;
 	case '\'': q = 1; break;
 	case ' ': q = 1; break;
+	case '!': q = 1; break;
 	}
 	if (q){
 	    char q = '\\';
@@ -788,7 +789,7 @@ void do_subshell_chdir (char *directory, int do_update, int reset_prompt)
     subshell_state = RUNNING_COMMAND;
     feed_subshell (QUIETLY, FALSE);
     
-    if (subshell_alive && strcmp (subshell_cwd, cpanel->cwd))
+    if (subshell_alive && strcmp (subshell_cwd, cpanel->cwd) && strcmp (cpanel->cwd, "."))
 	fprintf (stderr, "\rWarning: couldn't change to %s.\n\r", cpanel->cwd);
 
     if (reset_prompt)
@@ -893,10 +894,11 @@ static int feed_subshell (int how, int fail_on_error)
     wtime.tv_sec = 10;
     wtime.tv_usec = 0;
     
-    wptr = fail_on_error ? &wtime : NULL;
-	
-    for (;;)
+    for (wptr = fail_on_error ? &wtime : NULL;;)
     {
+	if (!subshell_alive)
+	    return FALSE;
+
 	/* {{{ Prepare the file-descriptor set and call `select' */
 
 	FD_ZERO (&read_set);
@@ -905,23 +907,17 @@ static int feed_subshell (int how, int fail_on_error)
 	if (how == VISIBLY)
 	    FD_SET (STDIN_FILENO, &read_set);
 
-	while (subshell_alive &&
-	       (s = select (FD_SETSIZE, &read_set, NULL, NULL, wptr) == -1))
-	    if (errno != EINTR)	  /* Despite using SA_RESTART, we */
-	    {			  /* still have to check for this */
-		tcsetattr (STDOUT_FILENO, TCSANOW, &shell_mode);
-		perror ("\n"__FILE__": select (FD_SETSIZE, &read_set...)");
-		exit (1);
-	    } else {
-		use_subshell = 0;
-		return FALSE;
-	    }
+	if (select (FD_SETSIZE, &read_set, NULL, NULL, wptr) == -1){
 
+	    /* Despite using SA_RESTART, we still have to check for this */
+	    if (errno == EINTR)
+		continue;	/* try all over again */
+	    tcsetattr (STDOUT_FILENO, TCSANOW, &shell_mode);
+	    perror ("\n"__FILE__": select (FD_SETSIZE, &read_set...)");
+	    exit (1);
+	}
 	/* }}} */
 
-	if (!subshell_alive)
-	    return FALSE;
-
 	/* From now on: block forever on the select call */
 	wptr = NULL;
 	
