@@ -1244,7 +1244,7 @@ static void chdir_sel (WTree *tree)
 	return;
     }
     change_panel ();
-    if (do_cd (tree->selected_ptr->name)){
+    if (do_cd (tree->selected_ptr->name, cd_exact)){
 	paint_panel (cpanel);
 	select_item (cpanel);
     } else {
