@@ -51,7 +51,6 @@
 #  include <sys/ioctl.h>
 #endif
 #include "fs.h"
-#include "fsusage.h"
 #include "mountlist.h"
 
 /* From dialog.h (not wanting to include it as
@@ -78,6 +77,8 @@ int easy_patterns = 1;
 int align_extensions = 1;
 int tilde_trunc = 1;
 
+struct mount_entry *mount_list;
+
 #ifndef HAVE_STRDUP
 char *strdup (char *s)
 {
@@ -497,7 +498,7 @@ char *file_date (time_t when)
     char   *p;
 
     p = ctime (&when);
-    p ? strcpy (timebuf, p) : p = "-----";
+    strcpy (timebuf, p ? p : "-----");
 #else
     strcpy (timebuf, ctime (&when));
 #endif
@@ -613,14 +614,6 @@ static void prepare_environment (void)
 }
 #endif
 
-/* Since ncurses uses a handler that automatically refreshes the */
-/* screen after a SIGCONT, and we don't want this behavior when */
-/* spawning a child, we save the original handler here */
-void save_stop_handler (void)
-{
-    sigaction (SIGTSTP, NULL, &startup_handler);
-}
-
 char *unix_error_string (int error_num)
 {
     static char buffer [256];
@@ -678,8 +671,6 @@ long blocks2kilos (int blocks, int bsize)
 	return blocks;
 }
 
-static struct mount_entry *mount_list;
-
 void init_my_statfs (void)
 {
 #ifndef NO_INFOMOUNT
@@ -687,124 +678,6 @@ void init_my_statfs (void)
 #endif
 }
 
-void my_statfs (struct my_statfs *myfs_stats, char *path)
-{
-    int i, len = 0;
-
-#ifndef NO_INFOMOUNT
-    struct mount_entry *entry = NULL;
-    struct mount_entry *temp = mount_list;
-    struct fs_usage fs_use;
-
-    while (temp){
-	i = strlen (temp->me_mountdir);
-	if (i > len && (strncmp (path, temp->me_mountdir, i) == 0))
-	    if (!entry || (path [i] == PATH_SEP || path [i] == 0)){
-		len = i;
-		entry = temp;
-	    }
-	temp = temp->me_next;
-    }
-
-    if (entry){
-	get_fs_usage (entry->me_mountdir, &fs_use);
-
-	myfs_stats->type = entry->me_dev;
-	myfs_stats->typename = entry->me_type;
-	myfs_stats->mpoint = entry->me_mountdir;
-	myfs_stats->device = entry->me_devname;
-	myfs_stats->avail = getuid () ? fs_use.fsu_bavail/2 : fs_use.fsu_bfree/2;
-	myfs_stats->total = fs_use.fsu_blocks/2;
-	myfs_stats->nfree = fs_use.fsu_ffree;
-	myfs_stats->nodes = fs_use.fsu_files;
-    } else
-#endif
-#ifdef _OS_NT
-	DWORD lpSectorsPerCluster, lpBytesPerSector, lpFreeClusters, lpClusters;
-       DWORD           lpMaximumComponentLength, dw, lpFileSystemFlags;
-       static char     lpVolumeNameBuffer[256], lpFileSystemNameBuffer[30];
-
-       GetDiskFreeSpace(NULL, &lpSectorsPerCluster, &lpBytesPerSector,
-			&lpFreeClusters, &lpClusters);
-
-       /* KBytes available */
-       myfs_stats->avail = lpSectorsPerCluster * lpBytesPerSector * lpFreeClusters / 1024;
-       
-       /* KBytes total */
-       myfs_stats->total = lpSectorsPerCluster * lpBytesPerSector * lpClusters / 1024; 
-       myfs_stats->nfree = lpFreeClusters;
-       myfs_stats->nodes = lpClusters;
-
-       GetVolumeInformation(NULL, lpVolumeNameBuffer, 255, NULL,
-			    &lpMaximumComponentLength, &lpFileSystemFlags,
-			    lpFileSystemNameBuffer, 30);
-
-       myfs_stats->mpoint = lpFileSystemNameBuffer;
-       myfs_stats->device = lpVolumeNameBuffer;
-
-
-       myfs_stats->type = GetDriveType(NULL);
-       switch (myfs_stats->type) {
-	   /*
-	    * mmm. DeviceIoControl may fail if you are not root case
-	    * F5_1Pt2_512,            5.25", 1.2MB,  512 bytes/sector
-	    * myfs_stats->typename = "5.25\" 1.2MB"; break; case
-	    * F3_1Pt44_512,           3.5",  1.44MB, 512 bytes/sector
-	    * myfs_stats->typename = "3.5\" 1.44MB"; break; case
-	    * F3_2Pt88_512,           3.5",  2.88MB, 512 bytes/sector
-	    * myfs_stats->typename = "3.5\" 2.88MB"; break; case
-	    * F3_20Pt8_512,           3.5",  20.8MB, 512 bytes/sector
-	    * myfs_stats->typename = "3.5\" 20.8MB"; break; case
-	    * F3_720_512,             3.5",  720KB,  512 bytes/sector
-	    * myfs_stats->typename = "3.5\" 720MB"; break; case
-	    * F5_360_512,             5.25", 360KB,  512 bytes/sector
-	    * myfs_stats->typename = "5.25\" 360KB"; break; case
-	    * F5_320_512,             5.25", 320KB,  512 bytes/sector
-	    * case F5_320_1024,       5.25", 320KB,  1024
-	    * bytes/sector myfs_stats->typename = "5.25\" 320KB"; break;
-	    * case F5_180_512,        5.25", 180KB,  512
-	    * bytes/sector myfs_stats->typename = "5.25\" 180KB"; break;
-	    * case F5_160_512,        5.25", 160KB,  512
-	    * bytes/sector myfs_stats->typename = "5.25\" 160KB"; break;
-	    * case RemovableMedia,    Removable media other than
-	    * floppy myfs_stats->typename = "Removable"; break; case
-	    * FixedMedia              Fixed hard disk media
-	    * myfs_stats->typename = "Hard Disk"; break; case Unknown:
-	    * Format is unknown
-	    */
-       case DRIVE_REMOVABLE:
-               myfs_stats->typename = "Removable";
-               break;
-       case DRIVE_FIXED:
-               myfs_stats->typename = "Hard Disk";
-               break;
-       case DRIVE_REMOTE:
-               myfs_stats->typename = "Networked";
-               break;
-       case DRIVE_CDROM:
-               myfs_stats->typename = "CD-ROM";
-               break;
-       case DRIVE_RAMDISK:
-               myfs_stats->typename = "RAM disk";
-               break;
-       default:
-               myfs_stats->typename = "unknown";
-               break;
-       };
-
-       if (0)
-#endif
-    {
-	myfs_stats->type = 0;
-	myfs_stats->mpoint = "unknown";
-	myfs_stats->device = "unknown";
-	myfs_stats->avail = 0;
-	myfs_stats->total = 0;
-	myfs_stats->nfree = 0;
-	myfs_stats->nodes = 0;
-    }
-}
-
 char *skip_separators (char *s)
 {
     for (;*s; s++)
