@@ -143,11 +143,31 @@ void delete_hook (Hook **hook_list, void (*hook_fn)(void *));
 int hook_present (Hook *hook_list, void (*hook_fn)(void *));
 
 #ifndef _OS_NT
-#define PATH_SEP '/'
-#define PATH_SEP_STR "/"
+#    define PATH_SEP '/'
+#    define PATH_SEP_STR "/"
 #else
-#define PATH_SEP '\\'
-#define PATH_SEP_STR "\\"
+#    define PATH_SEP '\\'
+#    define PATH_SEP_STR "\\"
+     void drive_cmd_a ();
+     void drive_cmd_b ();
+#endif
+
+/* taken from regex.c: */
+/* Jim Meyering writes:
+
+   "... Some ctype macros are valid only for character codes that
+   isascii says are ASCII (SGI's IRIX-4.0.5 is one such system --when
+   using /bin/cc or gcc but without giving an ansi option).  So, all
+   ctype uses should be through macros like ISPRINT...  If
+   STDC_HEADERS is defined, then autoconf has verified that the ctype
+   macros don't need to be guarded with references to isascii. ...
+   Defining isascii to 1 should let any compiler worth its salt
+   eliminate the && through constant folding."  */
+
+#if defined (STDC_HEADERS) || (!defined (isascii) && !defined (HAVE_ISASCII))
+#define ISASCII(c) 1
+#else
+#define ISASCII(c) isascii(c)
 #endif
 
 #endif
