@@ -57,9 +57,12 @@
 #include "util.h"
 #include "global.h"
 #include "fsusage.h"
+#include "fsusage.h"
 #include "mountlist.h"
+#include "mad.h"
 
 struct sigaction startup_handler;
+extern struct mount_entry *mount_list;
 
 #define UID_CACHE_SIZE 200
 #define GID_CACHE_SIZE 30
@@ -144,6 +147,14 @@ char *get_group (int gid)
     }
 }
 
+/* Since ncurses uses a handler that automatically refreshes the */
+/* screen after a SIGCONT, and we don't want this behavior when */
+/* spawning a child, we save the original handler here */
+void save_stop_handler (void)
+{
+    sigaction (SIGTSTP, NULL, &startup_handler);
+}
+
 int my_system (int as_shell_command, const char *shell, const char *command)
 {
     struct sigaction ignore, save_intr, save_quit, save_stop;
@@ -304,7 +315,7 @@ void check_error_pipe (void)
     if (old_error >= 0){
 	while (len < MAX_PIPE_SIZE)
 	{
-            struct fd_set select_set;
+            fd_set select_set;
             struct timeval timeout;
             FD_ZERO (&select_set);
             FD_SET (error_pipe[0], &select_set);
@@ -533,3 +544,109 @@ int gettimeofday( struct timeval * tv, struct timezone * tz)
     return 0;
 }
 #endif /* SCO_FLAVOR */
+
+#ifndef HAVE_PUTENV
+
+/* The following piece of code was copied from the GNU C Library */
+/* And is provided here for nextstep who lacks putenv */
+
+extern char **environ;
+
+#ifndef	HAVE_GNU_LD
+#define	__environ	environ
+#endif
+
+
+/* Put STRING, which is of the form "NAME=VALUE", in the environment.  */
+int
+putenv (const char *string)
+{
+    const char *const name_end = strchr (string, '=');
+    register size_t size;
+    register char **ep;
+    
+    if (name_end == NULL){
+	/* Remove the variable from the environment.  */
+	size = strlen (string);
+	for (ep = __environ; *ep != NULL; ++ep)
+	    if (!strncmp (*ep, string, size) && (*ep)[size] == '='){
+		while (ep[1] != NULL){
+		    ep[0] = ep[1];
+		    ++ep;
+		}
+		*ep = NULL;
+		return 0;
+	    }
+    }
+    
+    size = 0;
+    for (ep = __environ; *ep != NULL; ++ep)
+	if (!strncmp (*ep, string, name_end - string) &&
+	    (*ep)[name_end - string] == '=')
+	    break;
+	else
+	    ++size;
+    
+    if (*ep == NULL){
+	static char **last_environ = NULL;
+	char **new_environ = (char **) malloc ((size + 2) * sizeof (char *));
+	if (new_environ == NULL)
+	    return -1;
+	(void) memcpy ((void *) new_environ, (void *) __environ,
+		       size * sizeof (char *));
+	new_environ[size] = (char *) string;
+	new_environ[size + 1] = NULL;
+	if (last_environ != NULL)
+	    free ((void *) last_environ);
+	last_environ = new_environ;
+	__environ = new_environ;
+    }
+    else
+	*ep = (char *) string;
+    
+    return 0;
+}
+#endif /* !HAVE_PUTENV */
+
+void my_statfs (struct my_statfs *myfs_stats, char *path)
+{
+    int i, len = 0;
+
+#ifndef NO_INFOMOUNT
+    struct mount_entry *entry = NULL;
+    struct mount_entry *temp = mount_list;
+    struct fs_usage fs_use;
+
+    while (temp){
+	i = strlen (temp->me_mountdir);
+	if (i > len && (strncmp (path, temp->me_mountdir, i) == 0))
+	    if (!entry || (path [i] == PATH_SEP || path [i] == 0)){
+		len = i;
+		entry = temp;
+	    }
+	temp = temp->me_next;
+    }
+
+    if (entry){
+	get_fs_usage (entry->me_mountdir, &fs_use);
+
+	myfs_stats->type = entry->me_dev;
+	myfs_stats->typename = entry->me_type;
+	myfs_stats->mpoint = entry->me_mountdir;
+	myfs_stats->device = entry->me_devname;
+	myfs_stats->avail = getuid () ? fs_use.fsu_bavail/2 : fs_use.fsu_bfree/2;
+	myfs_stats->total = fs_use.fsu_blocks/2;
+	myfs_stats->nfree = fs_use.fsu_ffree;
+	myfs_stats->nodes = fs_use.fsu_files;
+    } else
+#endif
+    {
+	myfs_stats->type = 0;
+	myfs_stats->mpoint = "unknown";
+	myfs_stats->device = "unknown";
+	myfs_stats->avail = 0;
+	myfs_stats->total = 0;
+	myfs_stats->nfree = 0;
+	myfs_stats->nodes = 0;
+    }
+}
