@@ -119,6 +119,15 @@ static char hex_char[] = "0123456789ABCDEF";
 /* }}} */
 /* {{{ Clean-up functions */
 
+void
+close_view_file (WView *view)
+{
+    if (view->file != -1){
+	mc_close (view->file);
+	view->file = -1;
+    }
+}
+
 void free_file (WView *view)
 {
     int i;
@@ -127,7 +136,7 @@ void free_file (WView *view)
 
     if (view->mmaping){
 	mc_munmap (view->data, view->s.st_size);
-	mc_close (view->file);
+	close_view_file (view);
     } else 
 #endif /* HAVE_MMAP */
     {
@@ -142,8 +151,7 @@ void free_file (WView *view)
 	    /* Ignore errors because we don't want to hear about broken pipe */
 	    close_error_pipe (-1, NULL);
 	} else {
-	    if (view->file != -1)
-		mc_close (view->file);
+	    close_view_file (view);
 	}
     }
     /* Block_ptr may be zero if the file was a file with 0 bytes */
@@ -303,32 +311,32 @@ static char *load_view_file (WView *view, char *filename)
     }
     if (mc_fstat (view->file, &view->s) < 0){
 	set_view_init_error (view, 0);
-	mc_close (view->file);
+	close_view_file (view);
 	return copy_strings (" Can't stat file \n ",
 			     unix_error_string (errno), " ", 0);
     }
     if (S_ISDIR (view->s.st_mode) || S_ISSOCK (view->s.st_mode)
 	|| S_ISFIFO (view->s.st_mode)){
-	mc_close (view->file);
+	close_view_file (view);
 	return set_view_init_error (view, " Can't view: not a regular file ");
     }
 
 #ifdef JUST_FOR_CULTURAL_INFORMATION
     /* #ifed out because now we have growing buffers :-) */
     if (view->s.st_size < 1) {
-	mc_close (view->file);
+	close_view_file (view);
 	return set_view_init_error (view, " Can't view empty file ");
     }
 #endif
     if (view->s.st_size == 0){
 	/* Must be one of those nice files that grow (/proc) */
-	mc_close (view->file);
+	close_view_file (view);
 	return init_growing_view (view, 0, filename);
     }
     
     /* First, try to open a compressed file */
     if (!view->viewer_magic_flag && (is_gunzipable (view->file)) != 0){
-    	mc_close (view->file);
+	close_view_file (view);
 	cmd = copy_strings ("gzip -dc ", "'", filename, "'", 0);
 	return init_growing_view (view, cmd, filename);
     }
@@ -338,7 +346,7 @@ static char *load_view_file (WView *view, char *filename)
     view->data = mc_mmap (0, view->s.st_size, PROT_READ, MAP_FILE | MAP_SHARED,
 		          view->file, 0);
     if ((caddr_t) view->data == (caddr_t) -1){
-	mc_close (view->file);
+	close_view_file (view);
 /*	set_view_init_error (view, 0);
 	return copy_strings (" Can't mmap file \n ",
 			     unix_error_string (errno), " ", 0);*/
@@ -350,7 +358,7 @@ static char *load_view_file (WView *view, char *filename)
     view->mmaping = 1;
     return 0;
 #else
-    mc_close (view->file);
+    close_view_file (view);
     return init_growing_view (view, 0, filename);
 #endif
 }
@@ -1357,11 +1365,12 @@ static void search (WView *view, char *text,
 static long block_search (WView *view, char *buffer, int len)
 {
     int w = view->widget.cols - (view->have_frame * 2);
-    char *d = buffer;
+    char *d = buffer, b;
     long e;
 
     /* clear interrupt status */
     got_interrupt ();
+    enable_interrupt_key ();
     e = view->found_len ? view->search_start + 1 : view->search_start;
 
     search_update_steps (view);
@@ -1377,19 +1386,19 @@ static long block_search (WView *view, char *buffer, int len)
 	    if (got_interrupt ())
 		break;
 	}
-	disable_interrupt_key ();
-	if (*d == get_byte (view, e))
+	b = get_byte (view, e);
+	if (*d == b)
 	    d++;
 	else {
 	    e -= d - buffer;
 	    d = buffer;
 	}
-	enable_interrupt_key ();
 	if (d - buffer == len){
 	    disable_interrupt_key ();
 	    return e - len;
 	}
     }
+    disable_interrupt_key ();
     return 0;
 }
 
@@ -1446,6 +1455,7 @@ static void hex_search (WView *view, char *text)
 	    break;
 
 	case hex2:
+	    text [i] = toupper (text [i]);
 	    if ((p = strchr (hex_char, text [i])) != 0){
 		v |= (p - hex_char);
 		state = normal;
@@ -1970,7 +1980,7 @@ void view_adjust_size (Dlg_head *h)
 
     /* Look up the viewer and the buttonbar, we assume only two widgets here */
     view = (WView *) find_widget_type (h, (void *) &view_callback);
-    bar  = view->widget.parent->current->next->widget;
+    bar  = (WButtonBar *) view->widget.parent->current->next->widget;
     widget_set_size (&view->widget, 0, 0, LINES-1, COLS);
     widget_set_size (&bar->widget, LINES-1, 0, 1, COLS);
 }
