@@ -769,10 +769,14 @@ static int insert_char (WInput *in, int c_code)
     in->need_push = 1;
     if (strlen (in->buffer)+1 == in->current_max_len){
 	/* Expand the buffer */
-	char *narea = realloc(in->buffer, in->current_max_len + in->field_len);
+	char *narea = xmalloc(in->current_max_len + in->field_len, "string expansion");
 	if (narea){
+	    char *p = in->buffer;
+
+	    strcpy (narea, in->buffer);
 	    in->buffer = narea;
 	    in->current_max_len += in->field_len;
+	    free (p);
 	}
     }
     if (strlen (in->buffer)+1 < in->current_max_len){
