@@ -36,6 +36,7 @@
 #include <string.h>
 #include <unistd.h>
 #include <stdlib.h>
+#include <sys/types.h>
 #include <stdarg.h>
 #include <fcntl.h>
 #include <pwd.h>
@@ -139,6 +140,26 @@ char *ftpfs_get_host_and_username (char *path, char **host, char **user, int *po
     return get_host_and_username (path, host, user, port, 21, 1, pass);
 }
 
+static int
+select_on_two (int fd1, int fd2)
+{
+    fd_set set;
+    int v;
+    int maxfd = (fd1 > fd2 ? fd1 : fd2) + 1;
+
+    FD_ZERO(&set);
+    FD_SET(fd1, &set);
+    FD_SET(fd2, &set);
+    v = select (maxfd, &set, 0, 0, NULL);
+    if (v < 0)
+	return v;
+    if (FD_ISSET (fd1, &set))
+	return 1;
+    if (FD_ISSET (fd2, &set))
+	return 2;
+    return -1;
+}
+
 static int
 get_line (int sock, char *buf, int buf_len)
 {
@@ -287,8 +308,9 @@ static int command (struct ftpfs_connection *bucket, int wait_reply,
     if (status < 0){
 	disable_interrupt_key();
 	code = 421;
-	if (errno == EPIPE)
+	if (errno == EPIPE){
 	    got_sigpipe = 1;
+	}
 	return TRANSIENT;
     }
     if (wait_reply)
@@ -356,6 +378,9 @@ flush_all_directory(struct ftpfs_connection *bucket)
 static int 
 login_server (struct ftpfs_connection *bucket, char *netrcpass)
 {
+#if defined(HSC_PROXY)
+    char *proxypass, *proxyname;
+#endif
     char *pass;
     char *name;			/* login user name */
     
@@ -381,12 +406,70 @@ login_server (struct ftpfs_connection *bucket, char *netrcpass)
     
     /* Proxy server accepts: username@host-we-want-to-connect*/
     if (qproxy (bucket)){
+#if defined(HSC_PROXY)
+	char *p, *host;
+	int port;
+	ftpfs_get_host_and_username(ftpfs_proxy_host, &host, &proxyname,
+				    &port, &proxypass);
+	free(host);
+	p = copy_strings(" Proxy: Password required for ", proxyname, " ",
+			 NULL);
+	proxypass = input_dialog (p, "Password:", "");
+	free(p);
+	if (proxypass == NULL) {
+	    ftpfserrno = EPERM;
+	    return 0;
+	}
+	name = strdup(quser (bucket));
+#else
 	/* qhost(bucket) has: "!hostname" */
 	name = copy_strings (quser(bucket), "@", qhost(bucket)+1, 0);
+#endif
     } else 
 	name = strdup (quser (bucket));
     
     if (getreply (qsock(bucket), NULL, 0) == COMPLETE) {
+#if defined(HSC_PROXY)
+	if (qproxy(bucket)) {
+	    print_vfs_message("ftpfs: sending proxy login name");
+	    if (command (bucket, 1, "USER %s", proxyname) == CONTINUE) {
+		print_vfs_message("ftpfs: sending proxy user password");
+		if (command (bucket, 1, "PASS %s", proxypass) == COMPLETE)
+		    {
+			print_vfs_message("ftpfs: proxy authentication succeeded");
+			wipe_password (proxypass);
+			if (command (bucket, 1, "SITE %s", qhost(bucket)+1) ==
+			    COMPLETE) {
+			    print_vfs_message("ftpfs: connected to %s", qhost(bucket)+1);
+			}
+			else {
+			    bucket->failed_on_login = 1;
+			    wipe_password (proxypass);
+			    wipe_password (pass);
+			    free (proxyname);
+			    free (name);
+			    return 0;
+			}
+		    }
+		else {
+		    bucket->failed_on_login = 1;
+		    wipe_password (proxypass);
+		    wipe_password (pass);
+		    free (proxyname);
+		    free (name);
+		    return 0;
+		}
+	    }
+	    else {
+		bucket->failed_on_login = 1;
+		wipe_password (proxypass);
+		wipe_password (pass);
+		free (proxyname);
+		free (name);
+		return 0;
+	    }
+	}
+#endif
 	print_vfs_message("ftpfs: sending login name");
         if ((code = command (bucket, WAIT_REPLY, "USER %s", name)) == CONTINUE) {
 	    print_vfs_message("ftpfs: sending user password");
@@ -453,7 +536,8 @@ ftpfs_open_socket(struct ftpfs_connection *bucket)
     struct   hostent *hp;
     int      my_socket;
     char     *host;
-
+    int      port;
+    
     /* Use a proxy host? */
     host = qhost(bucket);
 
@@ -465,7 +549,11 @@ ftpfs_open_socket(struct ftpfs_connection *bucket)
     /* Hosts to connect to that start with a ! should use proxy */
     if (*host == '!' && ftpfs_proxy_host){
 	bucket->use_proxy = 1;
+#if defined(HSC_PROXY)
+	ftpfs_get_proxy_host_and_port (ftpfs_proxy_host, &host, &port);
+#else
 	host = ftpfs_proxy_host;
+#endif
     }
     
     /* Get host address */
@@ -486,7 +574,12 @@ ftpfs_open_socket(struct ftpfs_connection *bucket)
 	       hp->h_length);
     }
 
-    server_address.sin_port = htons (qport(bucket));
+#if defined(HSC_PROXY)
+#define THE_PORT qproxy(bucket) ? port : qport (bucket)
+#else
+#define THE_PORT qport(bucket)
+#endif
+    server_address.sin_port = htons (THE_PORT);
 
     /* Connect */
     if ((my_socket = socket (AF_INET, SOCK_STREAM, 0)) < 0) {
@@ -528,6 +621,14 @@ open_command_connection (char *host, char *user, int port, char *netrcpass)
 	ftpfserrno = ENOMEM;
 	return NULL;
     }
+#ifdef HAVE_MAD
+    {
+	extern void *watch_free_pointer;
+
+	if (!watch_free_pointer)
+	    watch_free_pointer = host;
+    }
+#endif
     qhost(bucket) = host;
     quser(bucket) = user;
     qcdir(bucket) = NULL;
@@ -540,7 +641,6 @@ open_command_connection (char *host, char *user, int port, char *netrcpass)
     bucket->lock = 0;
     bucket->use_proxy = 0;
     bucket->use_source_route = source_route;
-    retry_seconds = 0;
     
     if ((qdcache(bucket) = linklist_init()) == NULL) {
 	ftpfserrno = ENOMEM;
@@ -550,6 +650,7 @@ open_command_connection (char *host, char *user, int port, char *netrcpass)
 	return NULL;
     }
     
+    retry_seconds = 0;
     do { 
 	bucket->failed_on_login = 0;
 
@@ -559,20 +660,29 @@ open_command_connection (char *host, char *user, int port, char *netrcpass)
 	    return NULL;
 	}
 
-	if (!login_server(bucket, netrcpass)) {
-	    if (!bucket->failed_on_login){
+	if (login_server(bucket, netrcpass)) {
+	    /* Logged in, no need to retry the connection */
+	    break;
+	} else {
+	    if (bucket->failed_on_login){
+		/* Close only the socket descriptor */
+		close (qsock (bucket));
+	    } else {
 		ftpfs_connection_destructor (bucket);
+		free (bucket);
 		return NULL;
-	    }
+	    } 
 	    if (ftpfs_retry_seconds){
 		retry_seconds = ftpfs_retry_seconds;
 		enable_interrupt_key ();
 		for (count_down = retry_seconds; count_down; count_down--){
-		    print_vfs_message ("Waiting to retry... %d\n", count_down);
+		    print_vfs_message ("Waiting to retry... %d (Control-C to cancel)\n", count_down);
 		    sleep (1);
 		    if (got_interrupt ()){
 			retry_seconds = 0;
-			break;
+			disable_interrupt_key ();
+			free (bucket);
+			return NULL;
 		    }
 		}
 		disable_interrupt_key ();
@@ -593,14 +703,26 @@ open_command_connection (char *host, char *user, int port, char *netrcpass)
     return bucket;
 }
 
+#if defined(HSC_PROXY)
+int  ftpfs_get_proxy_host_and_port (char *proxy, char **host, int *port)
+{
+#define HSC_DEFAULT_PORT "9875"
+    char *user, *pass;
+    return get_host_and_username(proxy, host, &user, port, HSC_DEFAULT_PORT, 1,
+				 &pass);
+    free(user);
+    free(pass);
+}
+#endif
 static int
 is_connection_closed(struct ftpfs_connection *bucket)
 {
-    struct fd_set rset;
+    fd_set rset;
     struct timeval t;
 
-    if (got_sigpipe)
+    if (got_sigpipe){
 	return 1;
+    }
     t.tv_sec = 0;
     t.tv_usec = 0;
     FD_ZERO(&rset);
@@ -960,6 +1082,41 @@ done:
     getreply(qsock(bucket), NULL, 0);
 }
 
+static int
+get_line_interruptible (char *buffer, int size, int fd)
+{
+    int n;
+    int i = 0;
+
+    for (i = 0; i < size-1; i++) {
+	n = read (fd, buffer+i, 1);
+	if (n == -1 && errno == EINTR){
+	    buffer [i] = 0;
+	    return EINTR;
+	}
+	if (n == 0){
+	    buffer [i] = 0;
+	    return 0;
+	}
+	if (buffer [i] == '\n'){
+	    buffer [i] = 0;
+	    return 1;
+	}
+    }
+    buffer [size-1] = 0;
+    return 0;
+}
+
+/* Return true if path is the same directoy as the one we are on now */
+static int
+ftpfs_same_dir (char *path, struct ftpfs_connection *bucket)
+{
+    if (!qcdir (bucket))
+	return 0;
+    if (strcmp (path, qcdir (bucket)) == 0)
+	return 1;
+    return 0;
+}
 
 static struct ftpfs_dir *
 retrieve_dir(struct ftpfs_connection *bucket, char *remote_path)
@@ -1021,21 +1178,33 @@ retrieve_dir(struct ftpfs_connection *bucket, char *remote_path)
     sock = open_data_connection (bucket, "LIST -la", ".", 0);
     if (sock == -1)
 	goto error_3;
+
+#ifdef OLD_READ
+#define close_this_sock(x,y) fclose (x)
     fp = fdopen(sock, "r");
     if (fp == NULL) {
 	close(sock);
         goto error_2;
     }
-
+#else
+#define close_this_sock(x,y) close (y)
+#endif
+    
     /* Clear the interrupt flag */
     enable_interrupt_key ();
     
     errno = 0;
+#ifdef OLD_READ
     while (fgets (buffer, sizeof (buffer), fp) != NULL) {
-
 	if (got_intr = got_interrupt ())
 	    break;
-	
+#else
+    while ((got_intr = get_line_interruptible (buffer, sizeof (buffer), sock)) != EINTR){
+#endif
+	int eof = got_intr == 0;
+
+	if (buffer [0] == 0 && eof)
+	    break;
 	fe = xmalloc(sizeof(struct ftpentry), "struct ftpentry");
 	fe->freshly_created = 0;
 	if (fe == NULL) {
@@ -1057,16 +1226,22 @@ retrieve_dir(struct ftpfs_connection *bucket, char *remote_path)
 	}
 	else
 	    free(fe);
+	if (eof)
+	    break;
     }
     if (got_intr){
 	disable_interrupt_key();
 	print_vfs_message("ftpfs: reading FTP directory interrupt by user");
+#ifdef OLD_READ
 	ftpfs_abort(bucket, fileno(fp));
-	fclose(fp);
+#else
+	ftpfs_abort(bucket, sock);
+#endif
+	close_this_sock(fp, sock);
 	ftpfserrno = EINTR;
 	goto error_3;
     }
-    fclose(fp);
+    close_this_sock(fp, sock);
     disable_interrupt_key();
     if (getreply (qsock (bucket), NULL, 0) != COMPLETE) {
 	ftpfserrno = EIO;
@@ -1086,7 +1261,7 @@ retrieve_dir(struct ftpfs_connection *bucket, char *remote_path)
     return dcache;
 error_1:
     disable_interrupt_key();
-    fclose(fp);
+    close_this_sock(fp, sock);
 error_2:
     getreply(qsock(bucket), NULL, 0);
 error_3:
@@ -1243,15 +1418,9 @@ int ftpfs_ctl (void *data, int ctlop, int arg)
 
 	    enable_interrupt_key ();
             if (!remoten) {
-		fd_set set;
-		int v;
+		int v = select_on_two (remotesock, 0);
 		
-		FD_ZERO (&set);
-		FD_SET (remotesock, &set);
-		FD_SET (0, &set);
-		v = select (remotesock+1, &set, 0, 0, NULL);
-
-		if (((v < 0) && (errno == EINTR)) || (FD_ISSET (0, &set))){
+		if (((v < 0) && (errno == EINTR)) || v == 0){
 		    disable_interrupt_key ();
 		    return MCERR_DATA_ON_STDIN;
 		}
@@ -1948,24 +2117,26 @@ static int ftpfs_chdir (char *path)
 {
     char *remote_path;
     struct ftpfs_connection *bucket;
-    int r;
+    int r = COMPLETE;
 
  retry:
     if (!(remote_path = ftpfs_get_path(&bucket, path)))
 	return -1;
     got_sigpipe = 0;
     r = command (bucket, WAIT_REPLY, "CWD %s", remote_path);
-    if (got_sigpipe)
+    if (got_sigpipe){
 	goto retry;
-
+    }
+    
     if (r != COMPLETE) {
 	ftpfserrno = EIO;
-    	free(remote_path);
+	free(remote_path);
     } else {
-        if (qcdir(bucket))
+	if (qcdir(bucket))
 	    free(qcdir(bucket));
 	qcdir(bucket) = remote_path;
     }
+
     vfs_add_noncurrent_stamps (&ftpfs_vfs_ops, (vfsid) bucket, NULL);
     return r == COMPLETE ? 0 : -1;
 }
