@@ -64,6 +64,7 @@ static mcfs_open_connections = 0;
 struct {
     char *host;
     char *user;
+    char *home;
     int  sock;
     int  port;
 } mcfs_connections [MCFS_MAX_CONNECTIONS];
@@ -87,10 +88,12 @@ void mcfs_free_bucket (int bucket)
 {
     free (mcfs_connections [bucket].host);
     free (mcfs_connections [bucket].user);
+    free (mcfs_connections [bucket].home);
 
     /* Set all the fields to zero */
     mcfs_connections [bucket].host =
-	mcfs_connections [bucket].user = 0;
+	mcfs_connections [bucket].user =
+	    mcfs_connections [bucket].home = 0;
     mcfs_connections [bucket].sock = 0;
 }
 
@@ -254,6 +257,7 @@ static int mcfs_open_link (char *host, char *user, int *port, char *netrcpass)
     mcfs_open_connections++;
     mcfs_connections [bucket].host = strdup (host);
     mcfs_connections [bucket].user = strdup (user);
+    mcfs_connections [bucket].home = 0;
     mcfs_connections [bucket].port = *port;
     mcfs_connections [bucket].sock = sock;
 
@@ -402,17 +406,24 @@ static int mcfs_rpc_path_int_int (int command, char *path, int n1, int n2)
 char *mcfs_gethome (char *servername)
 {
     char *remote_file;
-    int  sock;
     char *buffer;
+    int  sock, i;
 
     if (!(remote_file = mcfs_get_path (&sock, servername)))
 	return 0;
     free (remote_file);
-    rpc_send (sock, RPC_INT, MC_GETHOME, RPC_END);
-    if (0 == rpc_get (sock, RPC_STRING, &buffer, RPC_END)) {
-        return strdup ("/");
-    }
-    return buffer;
+    for (i = 0; i < MCFS_MAX_CONNECTIONS; i++)
+	if (mcfs_connections[i].sock == sock)
+	    if (mcfs_connections[i].home)
+		return strdup (mcfs_connections[i].home);
+	    else {
+		rpc_send (sock, RPC_INT, MC_GETHOME, RPC_END);
+		if (0 == rpc_get (sock, RPC_STRING, &buffer, RPC_END))
+		    return strdup ("/");
+		mcfs_connections[i].home = buffer;
+		return strdup (buffer);
+	    }
+    return 0;
 }
 
 char *mcfs_getupdir (char *servername)
