@@ -78,6 +78,9 @@
 
 /* Authentication include files */
 #include <pwd.h>
+#ifdef HAVE_PAM
+#    include <security/pam_misc.h>
+#endif
 
 #include "../src/fs.h"
 #include "../src/mem.h"
@@ -128,6 +131,7 @@ int debug = 1;
 void send_status (int status, int errno_number)
 {
     rpc_send (msock, RPC_INT, status, RPC_INT, errno_number, RPC_END);
+    errno = 0;
 }
 
 /* }}} */
@@ -189,21 +193,23 @@ void do_write (void)
 
 void do_lseek (void)
 {
-    int handle, offset, whence;
+    int handle, offset, whence, status;
 
     rpc_get (msock,
 	     RPC_INT, &handle,
 	     RPC_INT, &offset,
 	     RPC_INT, &whence, RPC_END);
-    send_status (lseek (handle, offset, whence), errno);
+    status = lseek (handle, offset, whence);
+    send_status (status, errno);
 }
 
 void do_close (void)
 {
-    int handle;
+    int handle, status;
     
     rpc_get (msock, RPC_INT, &handle, RPC_END);
-    send_status (close (handle), errno);
+    status = close (handle);
+    send_status (status, errno);
 }
 
 /* }}} */
@@ -323,14 +329,18 @@ void do_stat ()
 static struct {
     int used;
     DIR *dirs [OPENDIR_HANDLES];
+    char *names [OPENDIR_HANDLES];
 } mcfs_DIR;
 
 void close_handle (int handle)
 {
-    mcfs_DIR.used--;
+    if (mcfs_DIR.used > 0) mcfs_DIR.used--;
     if (mcfs_DIR.dirs [handle])
 	closedir (mcfs_DIR.dirs [handle]);
+    if (mcfs_DIR.names [handle])
+	free (mcfs_DIR.names [handle]);
     mcfs_DIR.dirs [handle] = 0;
+    mcfs_DIR.names [handle] = 0;
 }
 
 void do_opendir (void)
@@ -339,12 +349,13 @@ void do_opendir (void)
     char *arg;
     DIR *p;
 
+    rpc_get (msock, RPC_STRING, &arg, RPC_END);
+
     if (mcfs_DIR.used == OPENDIR_HANDLES){
-	send_status (-1, errno);	/* Error */
+	send_status (-1, ENFILE);	/* Error */
+	free (arg);
 	return;
     }
-
-    rpc_get (msock, RPC_STRING, &arg, RPC_END);
     
     handle = -1;
     for (i = 0; i < OPENDIR_HANDLES; i++){
@@ -355,7 +366,10 @@ void do_opendir (void)
     }
 
     if (handle == -1){
-	send_status (-1, errno);
+	send_status (-1, EMFILE);
+	free (arg);
+	if (!inetd_started)
+	    fprintf (stderr, "OOPS! you have found a bug in mc - do_opendir()!\n");
 	return;
     }
 
@@ -363,15 +377,16 @@ void do_opendir (void)
     p = opendir (arg);
     if (p){
 	mcfs_DIR.dirs [handle] = p;
+	mcfs_DIR.names [handle] = arg;
 	    mcfs_DIR.used ++;
 
 	/* Because 0 is an error value */
 	rpc_send (msock, RPC_INT, handle+1, RPC_INT, 0, RPC_END);
 
-    } else
+    } else {
 	send_status (-1, errno);
-    
-    free (arg);
+	free (arg);
+    }
 }
 
 /* Sends the complete directory listing, as well as the stat information */
@@ -379,7 +394,8 @@ void do_readdir (void)
 {
     struct dirent *dirent;
     struct stat st;
-    int    handle, n;
+    int    handle, n, dnamelen;
+    char   *fname = 0;
 
     rpc_get (msock, RPC_INT, &handle, RPC_END);
     
@@ -390,18 +406,21 @@ void do_readdir (void)
 
     /* We incremented it in opendir */
     handle --;
+    dnamelen = strlen (mcfs_DIR.names [handle]);
 
     while ((dirent = readdir (mcfs_DIR.dirs [handle]))){
 	int length = NLENGTH (dirent);
 
 	rpc_send (msock, RPC_INT, length, RPC_END);
 	rpc_send (msock, RPC_BLOCK, length, dirent->d_name, RPC_END);
-	n = lstat (dirent->d_name, &st);
+	fname = malloc (dnamelen + length + 2);
+	strcat (strcat (strcpy (fname, mcfs_DIR.names [handle]), "/"), dirent->d_name);
+	n = lstat (fname, &st);
 	send_status (n, errno);
+	free (fname);
 	if (n >= 0)
 	    send_stat_info (&st);
     }
-    close_handle (handle);
     rpc_send (msock, RPC_INT, 0, RPC_END);
 }
 
@@ -420,42 +439,48 @@ void do_closedir (void)
 void do_chdir (void)
 {
     char *file;
+    int  status;
 
     rpc_get (msock, RPC_STRING, &file, RPC_END);
     
-    send_status (chdir (file), errno);
+    status = chdir (file);
+    send_status (status, errno);
     free (file);
 }
 
 void do_rmdir (void)
 {
     char *file;
+    int  status;
 
     rpc_get (msock, RPC_STRING, &file, RPC_END);
     
-    send_status (rmdir (file), errno);
+    status = rmdir (file);
+    send_status (status, errno);
     free (file);
 }
 
 void do_mkdir (void)
 {
     char *file;
-    int  mode;
+    int  mode, status;
 
     rpc_get (msock, RPC_STRING, &file, RPC_INT, &mode, RPC_END);
     
-    send_status (mkdir (file, mode), errno);
+    status = mkdir (file, mode);
+    send_status (status, errno);
     free (file);
 }
 
 void do_mknod (void)
 {
     char *file;
-    int  mode, dev;
+    int  mode, dev, status;
 
     rpc_get (msock, RPC_STRING, &file, RPC_INT, &mode, RPC_INT, &dev, RPC_END);
     
-    send_status (mknod (file, mode, dev), errno);
+    status = mknod (file, mode, dev);
+    send_status (status, errno);
     free (file);
 }
 
@@ -478,35 +503,43 @@ void do_readlink (void)
 void do_unlink (void)
 {
     char *file;
+    int  status;
     
     rpc_get (msock, RPC_STRING, &file, RPC_END);
-    send_status (unlink (file), errno);
+    status = unlink (file);
+    send_status (status, errno);
     free (file);
 }
 
 void do_rename (void)
 {
     char *f1, *f2;
+    int  status;
     
     rpc_get (msock, RPC_STRING, &f1, RPC_STRING, &f2, RPC_END);
-    send_status (rename (f1, f2), errno);
+    status = rename (f1, f2);
+    send_status (status, errno);
     free (f1); free (f2);
 }
 
 void do_symlink (void)
 {
     char *f1, *f2;
+    int  status;
     
     rpc_get (msock, RPC_STRING, &f1, RPC_STRING, &f2, RPC_END);
-    send_status (symlink (f1, f2), errno);
+    status = symlink (f1, f2);
+    send_status (status, errno);
     free (f1); free (f2);
 }
 
 void do_link (void)
 {
     char *f1, *f2;
+    int  status;
     
     rpc_get (msock, RPC_STRING, &f1, RPC_STRING, &f2, RPC_END);
+    status = link (f1, f2);
     send_status (link (f1, f2), errno);
     free (f1); free (f2);
 }
@@ -529,28 +562,116 @@ void do_getupdir (void)
 void do_chmod (void)
 {
     char *file;
-    int  mode;
+    int  mode, status;
     
     rpc_get (msock, RPC_STRING, &file, RPC_INT, &mode, RPC_END);
-    send_status (chmod (file, mode), errno);
+    status = chmod (file, mode);
+    send_status (status, errno);
     free (file);
 }
 
 void do_chown (void)
 {
     char *file;
-    int  owner, group;
+    int  owner, group, status;
     
     rpc_get (msock, RPC_STRING, &file,RPC_INT, &owner, RPC_INT,&group,RPC_END);
-    send_status (chown (file, owner, group), errno);
+    status = chown (file, owner, group);
+    send_status (status, errno);
     free (file);
 }
 
-void do_quit ()
+void do_quit (int result)
 {
-    exit (1);
+#ifdef HAVE_PMAP_SET
+    pmap_unset (RPC_PROGNUM, RPC_PROGVER);
+#endif
+    exit (result);
 }
 
+#ifdef HAVE_PAM
+
+struct user_pass {
+    char *username;
+    char *password;
+};
+
+int
+mc_pam_conversation (int messages, const struct pam_message **msg,
+		     struct pam_response **resp, void *appdata_ptr)
+{
+    struct pam_response *r;
+    struct user_pass *up = appdata_ptr;
+    int status;
+
+    r = (struct pam_response *) malloc (sizeof (struct pam_response) * messages);
+    if (!r)
+	return PAM_CONV_ERR;
+    *resp = r;
+    
+    for (status = PAM_SUCCESS; messages--; msg++){
+	switch ((*msg)->msg_style){
+
+	case PAM_PROMPT_ECHO_ON: 
+	    r->resp = strdup (up->username);
+	    r->resp_retcode  = PAM_SUCCESS;
+	    r++;
+	    break;
+
+	case PAM_PROMPT_ECHO_OFF:
+	    r->resp = strdup (up->password);
+	    r->resp_retcode = PAM_SUCCESS;
+	    r++;
+	    break;
+
+	case PAM_ERROR_MSG:
+	    status = PAM_CONV_ERR;
+	    break;
+
+	case PAM_TEXT_INFO:
+	    break;
+
+	}
+    }
+    if (status != PAM_SUCCESS)
+	*resp = NULL;
+    return status;
+}
+
+static struct pam_conv conv = { &mc_pam_conversation, NULL };
+			     
+
+/* Return 0 if authentication failed, 1 otherwise */
+int
+mc_pam_auth (char *username, char *password)
+{
+    pam_handle_t *pamh;
+    struct user_pass up;
+    int status;
+    int i = 0;
+    
+    up.username = username;
+    up.password = password;
+    conv.appdata_ptr = &up;
+    
+    if ((status = pam_start("mcserv", username, &conv, &pamh)) != PAM_SUCCESS)
+	goto failed_pam;
+    if ((status = pam_authenticate (pamh, 0)) != PAM_SUCCESS)
+	goto failed_pam;
+    if ((status = pam_acct_mgmt (pamh, 0)) != PAM_SUCCESS)
+	goto failed_pam;
+    if ((status = pam_setcred (pamh, PAM_CRED_ESTABLISH)) != PAM_SUCCESS)
+	goto failed_pam;
+    pam_end (pamh, status);
+    return 0;
+    
+failed_pam:
+    pam_end (pamh, status);
+    return 1;
+}
+
+#else /* Code for non-PAM authentication */
+    
 /* Keep reading until we find a \n */
 static int next_line (int socket)
 {
@@ -665,29 +786,37 @@ int do_classic_auth (char *username, char *password)
     endpwent ();
     return ret;
 }
+#endif /* non-PAM authentication */
 
 /* Try to authenticate the user based on:
+   - PAM if the system has it, else it checks:
    - pwdauth if the system supports it.
    - conventional auth (check salt on /etc/passwd, crypt, and compare
-   - try to contact the local ftp server and login
+   - try to contact the local ftp server and login (if -f flag used)
 */
-int do_auth (char *username, char *password)
+int
+do_auth (char *username, char *password)
 {
     int auth = 0;
     struct passwd *this;
     
     if (strcmp (username, "anonymous") == 0)
 	username = "ftp";
-    
+
+#ifdef HAVE_PAM
+    if (mc_pam_auth (username, password) == 0)
+	auth = 1;
+#else /* if there is no pam */
 #ifdef HAVE_PWDAUTH
     if (pwdauth (username, password) == 0)
 	auth = 1;
-#endif
     else
+#endif
     if (do_classic_auth (username, password))
 	auth = 1;
     else if (ftp)
 	auth = do_ftp_auth (username, password);
+#endif /* not pam */
     
     if (!auth)
         return 0;
@@ -792,10 +921,10 @@ void do_login ()
     rpc_send (msock, RPC_INT, MC_NEED_PASSWORD, RPC_END);
     rpc_get (msock, RPC_INT, &result, RPC_END);
     if (result == MC_QUIT)
-	exit (0);
+	do_quit (0);
     if (result != MC_PASS){
 	if (verbose) printf ("do_login: Unknown response: %d\n", result);
-	exit (0);
+	do_quit (0);
     }
     rpc_get (msock, RPC_STRING, &password, RPC_END);
     logged_in = do_auth (username, password);
@@ -845,7 +974,7 @@ void exec_command (int command)
 {
     if (commands [command].command == 0){
 	fprintf (stderr, "Got unknown command: %d\n", command);
-	exit (1);
+	do_quit (1);
     }
     if (verbose) printf ("Command: %s\n", commands [command].command);
     (*commands [command].callback)();
@@ -866,7 +995,7 @@ void check_version ()
 void tcp_invalidate_socket (int sock)
 {
     printf ("Connection closed\n");
-    exit (1);
+    do_quit (1);
 }
 
 void server (int sock)
@@ -968,7 +1097,7 @@ int get_port_number ()
     if (port == -1){
 	if (geteuid () == 0){
 	    fprintf (stderr, "Could not bind the server on a reserved port\n");
-	    exit (1);
+	    do_quit (1);
 	}
 	port = 0;
     }
@@ -1011,7 +1140,7 @@ int main (int argc, char *argv [])
     extern char *optarg;
     int c;
 
-    while ((c = getopt (argc, argv, "diqp:v")) != -1){
+    while ((c = getopt (argc, argv, "fdiqp:v")) != -1){
 	switch (c){
 	case 'd':
 	    isDaemon = 1;
@@ -1041,13 +1170,16 @@ int main (int argc, char *argv [])
 	case 'r':
 	    r_auth = 1;
 	    break;
-	    
+
 	default:
-	    fprintf (stderr, "Usage is: mcserv [-diqrv] [-p portnum]\n\n"
+	    fprintf (stderr, "Usage is: mcserv [options] [-p portnum]\n\n"
+		    "options are:\n"
 		    "-d  become a daemon (sets -q)\n"
 		    "-q  quiet mode\n"
-		    "-r  use rhost based authentication\n"
+		/*    "-r  use rhost based authentication\n" */
+#ifndef HAVE_PAM
 		    "-f  force ftp authentication\n"
+#endif
 		    "-v  verbose mode\n"
 		    "-p  to specify a port number to listen\n");
 	    exit (0);
@@ -1063,7 +1195,7 @@ int main (int argc, char *argv [])
 	printf ("Using port %d\n", portnum);
     if ((result = get_client (portnum)))
 	perror (result);
-    return 0;
+    do_quit (0);
 }
 
 /* This functions are not used */
