@@ -783,7 +783,14 @@ char *vfs_canon (char *path)
         
         /* There has to be a leading tar: or extfs prefix in the path */
 
-        istar = (vfs == &tarfs_vfs_ops);
+	if (vfs == &tarfs_vfs_ops)
+	    istar = 1;
+	else
+	    istar = 0;
+	
+        /* The original code does not compile right in NeXTStep:
+	 * istar = (vfs == &tarfs_vfs_ops);
+	 */
         if (istar)
             local_path = tarfs_analysis (path, &arc_name, 0);
         else {
@@ -1567,8 +1574,9 @@ int parse_ls_lga (char *p, struct stat *s, char **filename, char **linkname)
 	    free_and_return (0);
         tim.tm_mday = (int)atol (columns [idx++]);
     }
-    
-    if (columns [idx][2] != ':'){
+
+    /* Microsoft ftp server may send a non padded digit */
+    if (columns [idx][2] != ':' && columns [idx][1] != ':'){
 	/* There is a year */
         l = atol (columns [idx++]);
         if (l < 1900 || l > 3000)
@@ -1579,7 +1587,7 @@ int parse_ls_lga (char *p, struct stat *s, char **filename, char **linkname)
         tim.tm_sec = 0;
         year_supplied = 1;
     }
-    if (columns [idx][2] == ':'){
+    if (columns [idx][2] == ':' || columns [idx][1] == ':'){
         if (sscanf (columns [idx],
 		    "%2d:%2d", &tim.tm_hour, &tim.tm_min) != 2) {
 	    if (!year_supplied)
